@@ -4,8 +4,14 @@ use anyhow::{anyhow, bail};
 use derive_more::{Deref, Display};
 use indexmap::{IndexMap, indexmap};
 use itertools::Itertools;
-use serde::{Deserialize, Deserializer, Serialize};
-use slumber_util::Mapping;
+use serde::{
+    Deserialize, Serialize,
+    de::{self, value::StringDeserializer},
+};
+use slumber_util::{
+    Mapping, NEW_ISSUE_LINK,
+    yaml::{self, DeserializeYaml, Expected, LocatedError, SourcedYaml},
+};
 use std::{
     borrow::Cow,
     fmt::{self, Debug},
@@ -93,6 +99,7 @@ const KEY_MODIFIERS: Mapping<'static, KeyModifiers> = Mapping::new(&[
 #[derive(
     Copy, Clone, Debug, Display, Eq, PartialEq, Hash, Serialize, Deserialize,
 )]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[serde(rename_all = "snake_case")]
 pub enum Action {
     // vvvvv If adding a variant, make sure to update the docs vvvvv
@@ -208,9 +215,25 @@ impl Action {
     }
 }
 
+impl DeserializeYaml for Action {
+    fn expected() -> Expected {
+        Expected::String
+    }
+
+    fn deserialize(yaml: SourcedYaml) -> yaml::Result<Self> {
+        let location = yaml.location;
+        let s = yaml.try_into_string()?;
+        // Use serde's implementation for consistency with serialization
+        <Self as Deserialize>::deserialize(StringDeserializer::new(s)).map_err(
+            |error: de::value::Error| LocatedError::other(error, location),
+        )
+    }
+}
+
 /// One or more key combinations, which should correspond to a single action
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize)]
 #[cfg_attr(test, derive(PartialEq))]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[serde(transparent)]
 pub struct InputBinding(Vec<KeyCombination>);
 
@@ -238,6 +261,17 @@ impl Display for InputBinding {
     }
 }
 
+impl DeserializeYaml for InputBinding {
+    fn expected() -> Expected {
+        Expected::Sequence
+    }
+
+    fn deserialize(yaml: SourcedYaml) -> yaml::Result<Self> {
+        // Deserialize a list of key combinations
+        DeserializeYaml::deserialize(yaml).map(Self)
+    }
+}
+
 impl From<Vec<KeyCombination>> for InputBinding {
     fn from(combo: Vec<KeyCombination>) -> Self {
         Self(combo)
@@ -257,8 +291,9 @@ impl From<KeyCode> for InputBinding {
 }
 
 /// Key input sequence, which can trigger an action
-#[derive(Copy, Clone, Debug, Serialize, Deserialize)]
+#[derive(Copy, Clone, Debug, Serialize)]
 #[cfg_attr(test, derive(PartialEq))]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[serde(into = "String", try_from = "String")]
 pub struct KeyCombination {
     pub code: KeyCode,
@@ -371,12 +406,17 @@ impl From<KeyCombination> for String {
     }
 }
 
-/// For deserialization
-impl TryFrom<String> for KeyCombination {
-    type Error = anyhow::Error;
+/// Deserialize via FromStr
+impl DeserializeYaml for KeyCombination {
+    fn expected() -> Expected {
+        Expected::String
+    }
 
-    fn try_from(value: String) -> Result<Self, Self::Error> {
-        value.parse()
+    fn deserialize(yaml: SourcedYaml) -> yaml::Result<Self> {
+        let location = yaml.location;
+        let s = yaml.try_into_string()?;
+        s.parse()
+            .map_err(|error| LocatedError::other(error, location))
     }
 }
 
@@ -391,6 +431,7 @@ impl TryFrom<String> for KeyCombination {
 #[derive(Clone, Debug, Deref, Serialize)]
 #[cfg_attr(test, derive(PartialEq))]
 #[serde(transparent)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 pub struct InputMap(IndexMap<Action, InputBinding>);
 
 impl InputMap {
@@ -461,16 +502,17 @@ impl Default for InputMap {
     }
 }
 
-impl<'de> Deserialize<'de> for InputMap {
+impl DeserializeYaml for InputMap {
+    fn expected() -> Expected {
+        Expected::Mapping
+    }
+
     /// Deserialize an input map. First we deserialize the user's provided
     /// bindings, then we'll populate the map with the defaults so the consumer
     /// has access to all the bindings in one place
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
+    fn deserialize(yaml: SourcedYaml) -> yaml::Result<Self> {
         let user_bindings: IndexMap<Action, InputBinding> =
-            IndexMap::deserialize(deserializer)?;
+            DeserializeYaml::deserialize(yaml)?;
         Ok(Self::new(user_bindings))
     }
 }
@@ -492,11 +534,18 @@ fn parse_key_code(s: &str) -> anyhow::Result<KeyCode> {
 
 /// Convert key code to string. Inverse of parsing
 fn stringify_key_code(code: KeyCode) -> Cow<'static, str> {
-    // ASCII chars aren't in the mapping, they're handled specially
-    if let KeyCode::Char(c) = code {
+    if let Some(label) = KEY_CODES.get_label(code) {
+        // If it's mapped, use the mapped label
+        label.into()
+    } else if let KeyCode::Char(c) = code {
+        // Otherwise we hope it's an ASCII char
         c.to_string().into()
     } else {
-        KEY_CODES.get_label(code).into()
+        // Indicates a bug: something that isn't mapped should be
+        panic!(
+            "Unmapped key code {code:?}; \
+            this is a bug, please open an issue: {NEW_ISSUE_LINK}"
+        )
     }
 }
 
@@ -512,15 +561,15 @@ fn parse_key_modifier(s: &str) -> anyhow::Result<KeyModifiers> {
 
 /// Convert key modifier to string. Inverse of parsing
 fn stringify_key_modifier(modifier: KeyModifiers) -> Cow<'static, str> {
-    KEY_MODIFIERS.get_label(modifier).into()
+    // unwrap() is safe because all possible modifiers are mapped
+    KEY_MODIFIERS.get_label(modifier).unwrap().into()
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
     use rstest::rstest;
-    use serde_test::{Token, assert_de_tokens, assert_de_tokens_error};
-    use slumber_util::assert_err;
+    use slumber_util::{assert_err, yaml::deserialize_yaml};
     use terminput::{KeyEventKind, KeyEventState};
 
     #[rstest]
@@ -702,46 +751,37 @@ mod tests {
     /// that string/lists are both supported
     #[test]
     fn test_deserialize_input_binding() {
-        assert_de_tokens(
-            &InputBinding(vec![KeyCode::F(2).into(), KeyCode::F(3).into()]),
-            &[
-                Token::Seq { len: Some(2) },
-                Token::Str("f2"),
-                Token::Str("f3"),
-                Token::SeqEnd,
-            ],
+        assert_eq!(
+            deserialize_yaml::<InputBinding>(vec!["f2", "f3"].into()).unwrap(),
+            InputBinding(vec![KeyCode::F(2).into(), KeyCode::F(3).into()])
         );
 
-        assert_de_tokens_error::<InputBinding>(
-            &[Token::Seq { len: Some(1) }, Token::Str("no"), Token::SeqEnd],
-            "Invalid key code \"no\"; key combinations should be space-separated",
+        assert_err!(
+            deserialize_yaml::<InputBinding>(vec!["no"].into())
+                .map_err(LocatedError::into_error),
+            "Invalid key code \"no\""
         );
-        assert_de_tokens_error::<InputBinding>(
-            &[
-                Token::Seq { len: Some(1) },
-                Token::Str("shart f2"),
-                Token::SeqEnd,
-            ],
+        assert_err!(
+            deserialize_yaml::<InputBinding>(vec!["shart f2"].into())
+                .map_err(LocatedError::into_error),
             "Invalid key modifier \"shart\"; must be one of \
-             [\"shift\", \"alt\", \"ctrl\", \"super\", \"hyper\", \"meta\"]",
+             [\"shift\", \"alt\", \"ctrl\", \"super\", \"hyper\", \"meta\"]"
         );
-        assert_de_tokens_error::<InputBinding>(
-            &[
-                Token::Seq { len: Some(2) },
-                Token::Str("f2"),
-                Token::Str("cortl f3"),
-                Token::SeqEnd,
-            ],
+        assert_err!(
+            deserialize_yaml::<InputBinding>(vec!["f2", "cortl f3"].into())
+                .map_err(LocatedError::into_error),
             "Invalid key modifier \"cortl\"; must be one of \
-            [\"shift\", \"alt\", \"ctrl\", \"super\", \"hyper\", \"meta\"]",
+            [\"shift\", \"alt\", \"ctrl\", \"super\", \"hyper\", \"meta\"]"
         );
-        assert_de_tokens_error::<InputBinding>(
-            &[Token::Str("f3")],
-            "invalid type: string \"f3\", expected a sequence",
+        assert_err!(
+            deserialize_yaml::<InputBinding>("f3".into())
+                .map_err(LocatedError::into_error),
+            "Expected sequence, received \"f3\""
         );
-        assert_de_tokens_error::<InputBinding>(
-            &[Token::I64(3)],
-            "invalid type: integer `3`, expected a sequence",
+        assert_err!(
+            deserialize_yaml::<InputBinding>(3.into())
+                .map_err(LocatedError::into_error),
+            "Expected sequence, received `3`"
         );
     }
 
