@@ -6,64 +6,103 @@ use crate::{
         json::JsonTemplate,
         recipe_tree::{RecipeNode, RecipeTree},
     },
-    http::{HttpMethod, content_type::ContentType, query::Query},
-    template::{Identifier, Template, TemplateParseError},
+    http::HttpMethod,
 };
-use anyhow::Context;
-use derive_more::{Deref, Display, From, FromStr, Into};
+use derive_more::{Deref, Display, From, Into};
 use indexmap::IndexMap;
 use mime::Mime;
 use reqwest::header;
 use serde::{Deserialize, Serialize};
-use slumber_util::{ResultTraced, parse_yaml};
-use std::{fs::File, path::PathBuf, time::Duration};
+use slumber_template::{Template, TemplateParseError};
+use slumber_util::{ResultTraced, doc_link, yaml};
+use std::{iter, path::Path};
 use tracing::info;
 
 /// A collection of profiles, requests, etc. This is the primary Slumber unit
 /// of configuration.
-///
-/// This deliberately does not implement `Clone`, because it could potentially
-/// be very large. Instead, it's hidden behind an `Arc` by `CollectionFile`.
-#[derive(Debug, Default, Serialize, Deserialize)]
+#[derive(Debug, Default, Serialize)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(deny_unknown_fields)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[cfg_attr(
+    feature = "schema",
+    schemars(
+        // Allow any top-level property beginning with .
+        extend("patternProperties" = {
+            "^\\.": { "description": "Ignore any property beginning with `.`" }
+        }),
+        example = Collection::example(),
+    )
+)]
 pub struct Collection {
     /// Descriptive name for the collection
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub name: Option<String>,
-    #[serde(default, deserialize_with = "cereal::deserialize_profiles")]
+    /// Map of profiles, keyed by their unique IDs
     pub profiles: IndexMap<ProfileId, Profile>,
-    #[serde(default, deserialize_with = "cereal::deserialize_id_map")]
-    pub chains: IndexMap<ChainId, Chain>,
-    /// Internally we call these recipes, but to a user `requests` is more
-    /// intuitive
-    #[serde(default, rename = "requests")]
+    /// Map of requests and folders, keyed by their unique IDs. Folders allow
+    /// for nested maps of more requests and folders. All IDs must be unique
+    /// **throughout the entire tree**, not just at their level.
+    #[serde(rename = "requests")]
     pub recipes: RecipeTree,
 }
 
 impl Collection {
     /// Load collection from a file
-    pub fn load(path: &PathBuf) -> anyhow::Result<Self> {
+    pub fn load(path: &Path) -> anyhow::Result<Self> {
         info!(?path, "Loading collection file");
-
-        let load = || {
-            let file = File::open(path)?;
-            let collection = parse_yaml(&file)?;
-            Ok::<_, anyhow::Error>(collection)
-        };
-
-        load()
-            .context(format!(
-                "Error loading collection from `{}`",
-                path.display()
-            ))
+        yaml::deserialize_file(path)
+            .map_err(|error| {
+                // If this looks like a v3 collection, give some extra help
+                if error
+                    .chain()
+                    .filter_map(|error| error.downcast_ref::<yaml::Error>())
+                    .any(Self::is_v3_error)
+                {
+                    error.context(format!(
+                        "This looks like a collection from Slumber v3. \
+                        Migrate to v4 or downgrade your installation to 3.x.\
+                        \n{}",
+                        doc_link("other/v4_migration")
+                    ))
+                } else {
+                    error
+                }
+            })
             .traced()
     }
+
+    /// Load collection from a YAML string
+    pub fn parse(input: &str) -> anyhow::Result<Self> {
+        yaml::deserialize_str(input).traced()
+    }
+
+    /// Does the deserialization error lead us to believe the collection is in
+    /// the v3 format? If so we'll give the user an extra message suggesting an
+    /// upgrade to v4
+    fn is_v3_error(error: &yaml::Error) -> bool {
+        match error {
+            // Look for:
+            // - `chains` field
+            // - `<<` merge key
+            // - `!tag`
+            // - `chains.` in a template
+            yaml::Error::UnexpectedField(field) => field == "chains",
+            yaml::Error::UnsupportedMerge => true,
+            yaml::Error::Unexpected { actual, .. } => actual.starts_with("tag"),
+            yaml::Error::Other(error) => error
+                // Check if there's a template containing a chain reference
+                .downcast_ref::<TemplateParseError>()
+                .is_some_and(|error| error.to_string().contains("{{chains.")),
+            _ => false,
+        }
+    }
 }
 
 /// Mutually exclusive hot-swappable config group
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Debug, Serialize)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(deny_unknown_fields)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[cfg_attr(feature = "schema", schemars(example = Profile::example()))]
 pub struct Profile {
     #[serde(skip)] // This will be auto-populated from the map key
     pub id: ProfileId,
@@ -72,7 +111,8 @@ pub struct Profile {
     /// the TUI, select this profile by default from the list. Only one profile
     /// in the collection can be marked as default. This is enforced by a
     /// custom deserializer function.
-    #[serde(default)]
+    #[serde(skip_serializing_if = "cereal::is_false")] // Skip if default
+    #[cfg_attr(feature = "schema", schemars(default))]
     pub default: bool,
     pub data: IndexMap<String, Template>,
 }
@@ -116,6 +156,7 @@ impl slumber_util::Factory for Profile {
 )]
 #[deref(forward)]
 #[serde(transparent)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 pub struct ProfileId(String);
 
 #[cfg(any(test, feature = "test"))]
@@ -133,19 +174,18 @@ impl slumber_util::Factory for ProfileId {
 }
 
 /// A gathering of like-minded recipes and/or folders
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Debug, Serialize)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(deny_unknown_fields)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[cfg_attr(feature = "schema", schemars(example = Folder::example()))]
 pub struct Folder {
     #[serde(skip)] // This will be auto-populated from the map key
     pub id: RecipeId,
+    /// Display name
     pub name: Option<String>,
-    /// RECURSION. Use `requests` in serde to match the root field.
-    #[serde(
-        default,
-        deserialize_with = "cereal::deserialize_id_map",
-        rename = "requests"
-    )]
+    /// Child requests of this folder
+    // Use `requests` in serde to match the root collection field
+    #[serde(rename = "requests")]
     pub children: IndexMap<RecipeId, RecipeNode>,
 }
 
@@ -167,41 +207,66 @@ impl slumber_util::Factory for Folder {
     }
 }
 
-/// A definition of how to make a request. This is *not* called `Request` in
-/// order to distinguish it from a single instance of an HTTP request. And it's
-/// not called `RequestTemplate` because the word "template" has a specific
-/// meaning related to string interpolation.
-#[expect(clippy::unsafe_derive_deserialize)]
-#[derive(Debug, Serialize, Deserialize)]
+/// A definition of how to build an HTTP request. This is also commonly called
+/// "request" throughout Slumber documentation because that term is more common
+/// and intuitive.
+#[derive(Debug, Serialize)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(deny_unknown_fields)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+#[cfg_attr(feature = "schema", schemars(example = Recipe::example()))]
 pub struct Recipe {
     #[serde(skip)] // This will be auto-populated from the map key
     pub id: RecipeId,
-    #[serde(default = "cereal::persist_default")]
+    /// Should requests and responses of this recipe be persisted in the local
+    /// Slumber database?
+    /// [See docs](https://slumber.lucaspickering.me/book/user_guide/database.html)
+    /// for more info
+    #[serde(skip_serializing_if = "cereal::is_true")] // Skip if default
+    #[cfg_attr(feature = "schema", schemars(default = "persist_default"))]
     pub persist: bool,
+    /// Display name
     #[serde(skip_serializing_if = "Option::is_none")]
     pub name: Option<String>,
-    /// *Not* a template string because the usefulness doesn't justify the
-    /// complexity. This gives the user an immediate error if the method is
-    /// wrong which is helpful.
+    /// [HTTP request method](https://developer.mozilla.org/en-US/docs/Web/HTTP/Reference/Methods)
+    // *Not* a template string because the usefulness doesn't justify the
+    // complexity. This gives the user an immediate error if the method is
+    // wrong which is helpful.
     pub method: HttpMethod,
+    /// [HTTP request URL](https://developer.mozilla.org/en-US/docs/Learn_web_development/Howto/Web_mechanics/What_is_a_URL)
+    ///
+    /// Query parameters *can* be included here, but typically it's easier to
+    /// use the `query` field instead.
     pub url: Template,
+    /// HTTP request body
+    ///
+    /// - `type: json`: `application/json` body
+    /// - `type: form_urlencoded`: `application/x-www-form-urlencoded` body
+    /// - `type: form_multipart`: `multipart/form-data` body
+    /// - Any template can be given to define the literal request body
+    ///   text/bytes. In this case, consider including a
+    ///   [`Content-Type` header](https://developer.mozilla.org/en-US/docs/Web/HTTP/Reference/Headers/Content-Type)
+    ///   to tell the server what type of content you're sending.
+    ///
+    /// See individual variants for more details on usage.
     #[serde(skip_serializing_if = "Option::is_none")]
     pub body: Option<RecipeBody>,
+    /// HTTP authentication scheme
+    ///
+    /// - `type: basic`: [Basic authentication](https://swagger.io/docs/specification/v3_0/authentication/basic-authentication/)
+    /// - `type: bearer`: [Bearer authentication](https://swagger.io/docs/specification/v3_0/authentication/bearer-authentication/)
+    ///
+    /// See individual variants for more details on usage.
     #[serde(skip_serializing_if = "Option::is_none")]
     pub authentication: Option<Authentication>,
-    #[serde(
-        default,
-        with = "cereal::serde_query_parameters",
-        skip_serializing_if = "Vec::is_empty"
-    )]
-    pub query: Vec<(String, Template)>,
-    #[serde(
-        default,
-        deserialize_with = "cereal::deserialize_headers",
-        skip_serializing_if = "IndexMap::is_empty"
-    )]
+    /// A map of [URL query parameters](https://developer.mozilla.org/en-US/docs/Learn_web_development/Howto/Web_mechanics/What_is_a_URL#parameters).
+    /// Each value can either be a single value (`?foo=bar`) or multiple
+    /// (`?foo=bar&foo=baz`)
+    #[serde(skip_serializing_if = "IndexMap::is_empty")]
+    #[cfg_attr(feature = "schema", schemars(default))]
+    pub query: IndexMap<String, QueryParameterValue>,
+    /// A map of [HTTP request headers](https://developer.mozilla.org/en-US/docs/Web/HTTP/Reference/Headers)
+    #[serde(skip_serializing_if = "IndexMap::is_empty")]
+    #[cfg_attr(feature = "schema", schemars(default))]
     pub headers: IndexMap<String, Template>,
 }
 
@@ -223,6 +288,22 @@ impl Recipe {
             .and_then(|template| template.display().parse::<Mime>().ok())
             .or_else(|| self.body.as_ref()?.mime())
     }
+
+    /// Get a _flattened_ iterator over this recipe's query parameters. Any
+    /// parameter with multiple values will be flattened so the parameter name
+    /// appears multiple times, once with each value. Each tuple will include
+    /// the index of each value to distinguish repeated parameters. The index
+    /// is unique to each parameter; it resets to 0 for each new parameter. This
+    /// will respect all ordering from the original map.
+    pub fn query_iter(&self) -> impl Iterator<Item = (&str, usize, &Template)> {
+        self.query.iter().flat_map(|(k, v)| {
+            let iter: Box<dyn Iterator<Item = _>> = match v {
+                QueryParameterValue::One(value) => Box::new(iter::once(value)),
+                QueryParameterValue::Many(values) => Box::new(values.iter()),
+            };
+            iter.enumerate().map(move |(i, v)| (k.as_str(), i, v))
+        })
+    }
 }
 
 #[cfg(any(test, feature = "test"))]
@@ -236,7 +317,7 @@ impl slumber_util::Factory for Recipe {
             url: "http://localhost/url".into(),
             body: None,
             authentication: None,
-            query: Vec::new(),
+            query: IndexMap::new(),
             headers: IndexMap::new(),
         }
     }
@@ -253,6 +334,12 @@ impl slumber_util::Factory<&str> for Recipe {
     }
 }
 
+/// Default value for `Recipe::persist`
+#[cfg(feature = "schema")]
+fn persist_default() -> bool {
+    true
+}
+
 #[derive(
     Clone,
     Debug,
@@ -269,6 +356,7 @@ impl slumber_util::Factory<&str> for Recipe {
 )]
 #[deref(forward)]
 #[serde(transparent)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 pub struct RecipeId(String);
 
 #[cfg(any(test, feature = "test"))]
@@ -280,7 +368,7 @@ impl From<&str> for RecipeId {
 
 /// For rstest magic conversions
 #[cfg(any(test, feature = "test"))]
-impl FromStr for RecipeId {
+impl std::str::FromStr for RecipeId {
     type Err = ();
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
@@ -295,39 +383,50 @@ impl slumber_util::Factory for RecipeId {
     }
 }
 
-#[cfg(any(test, feature = "test"))]
-impl slumber_util::Factory for Chain {
-    fn factory((): ()) -> Self {
-        Self {
-            id: "chain1".into(),
-            source: ChainSource::Request {
-                recipe: RecipeId::factory(()),
-                trigger: Default::default(),
-                section: Default::default(),
-            },
-            sensitive: false,
-            selector: None,
-            selector_mode: SelectorMode::default(),
-            content_type: None,
-            trim: ChainOutputTrim::default(),
-        }
-    }
-}
-
 /// Shortcut for defining authentication method. If this is defined in addition
 /// to the `Authorization` header, that header will end up being included in the
 /// request twice.
 ///
 /// Type parameter allows this to be re-used for post-render purposes (with
 /// `T=String`).
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(rename_all = "snake_case", deny_unknown_fields)]
+#[serde(tag = "type", rename_all = "snake_case")]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 pub enum Authentication<T = Template> {
     /// `Authorization: Basic {username:password | base64}`
     Basic { username: T, password: Option<T> },
     /// `Authorization: Bearer {token}`
-    Bearer(T),
+    Bearer { token: T },
+}
+
+/// A value for a particular query parameter key
+#[derive(Clone, Debug, Serialize)]
+#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
+#[serde(untagged)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+pub enum QueryParameterValue {
+    /// The common case: `?foo=bar`
+    One(Template),
+    /// Multiple values for the same parameter. This will be represented by
+    /// repeating the parameter key: `?foo=bar&foo=baz`
+    Many(Vec<Template>),
+}
+
+#[cfg(any(test, feature = "test"))]
+impl From<&str> for QueryParameterValue {
+    fn from(value: &str) -> Self {
+        QueryParameterValue::One(value.into())
+    }
+}
+
+#[cfg(any(test, feature = "test"))]
+impl<const N: usize> From<[&str; N]> for QueryParameterValue {
+    fn from(values: [&str; N]) -> Self {
+        QueryParameterValue::Many(
+            values.into_iter().map(Template::from).collect(),
+        )
+    }
 }
 
 /// Template for a request body. `Raw` is the "default" variant, which
@@ -336,17 +435,26 @@ pub enum Authentication<T = Template> {
 /// HTTP engine uses the variant to determine not only how to serialize the
 /// body, but also other parameters of the request (e.g. the `Content-Type`
 /// header).
-#[derive(Debug)]
+#[derive(Debug, Serialize)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
+#[serde(tag = "type", content = "data", rename_all = "snake_case")]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 pub enum RecipeBody {
-    /// Plain string/bytes body
-    Raw(Template),
-    /// `application/json` body
+    /// `application/json` body. Value can be any JSON value, with strings being
+    /// interpreted as templates
     Json(JsonTemplate),
-    /// `application/x-www-form-urlencoded` fields. Values must be strings
+    /// `application/x-www-form-urlencoded` body. Value is a mapping of form
+    /// field name to value. Values are templates while field names are not.
+    /// Values must render to strings.
     FormUrlencoded(IndexMap<String, Template>),
-    /// `multipart/form-data` fields. Values can be binary
+    /// `multipart/form-data` body. Value is a mapping of form field
+    /// name to value. Values are templates while field names are not. Values
+    /// can render to any bytes.
     FormMultipart(IndexMap<String, Template>),
+    /// Plain string/bytes body. Must be the last variant to support untagged.
+    /// This captures any value that doesn't fit one of the above variants.
+    #[serde(untagged)]
+    Raw(Template),
 }
 
 impl RecipeBody {
@@ -385,191 +493,6 @@ impl From<&str> for RecipeBody {
     }
 }
 
-/// A chain is a means to data from one response in another request. The chain
-/// is the middleman: it defines where and how to pull the value, then recipes
-/// can use it in a template via `{{chains.<chain_id>}}`.
-#[derive(Debug, Serialize, Deserialize)]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(deny_unknown_fields)]
-pub struct Chain {
-    #[serde(skip)] // This will be auto-populated from the map key
-    pub id: ChainId,
-    pub source: ChainSource,
-    /// Mask chained value in the UI
-    #[serde(default)]
-    pub sensitive: bool,
-    /// Selector to extract a value from the response. This uses JSONPath
-    /// regardless of the content type. Non-JSON values will be converted to
-    /// JSON, then converted back.
-    pub selector: Option<Query>,
-    /// Control selector behavior relative to number of query results
-    #[serde(default)]
-    pub selector_mode: SelectorMode,
-    /// Hard-code the content type of the response. Only needed if a selector
-    /// is given and the content type can't be dynamically determined
-    /// correctly. This is needed if the chain source is not an HTTP
-    /// response (e.g. a file) **or** if the response's `Content-Type` header
-    /// is incorrect.
-    pub content_type: Option<ContentType>,
-    #[serde(default)]
-    pub trim: ChainOutputTrim,
-}
-
-/// Unique ID for a chain, provided by the user
-#[derive(
-    Clone,
-    Debug,
-    Deref,
-    Default,
-    Display,
-    Eq,
-    FromStr,
-    Hash,
-    PartialEq,
-    Serialize,
-    Deserialize,
-)]
-#[deref(forward)]
-#[serde(transparent)]
-#[cfg_attr(test, derive(proptest_derive::Arbitrary))]
-pub struct ChainId(#[deref(forward)] Identifier);
-
-impl<T: Into<Identifier>> From<T> for ChainId {
-    fn from(value: T) -> Self {
-        Self(value.into())
-    }
-}
-
-/// The source of data for a chain
-#[derive(Debug, Serialize, Deserialize)]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(rename_all = "snake_case", deny_unknown_fields)]
-pub enum ChainSource {
-    /// Run an external command to get a result
-    Command {
-        command: Vec<Template>,
-        stdin: Option<Template>,
-    },
-    /// Load from an environment variable
-    #[serde(rename = "env")]
-    Environment { variable: Template },
-    /// Load data from a file
-    File { path: Template },
-    /// Prompt the user for a value
-    Prompt {
-        /// Descriptor to show to the user
-        message: Option<Template>,
-        /// Default value for the shown textbox
-        default: Option<Template>,
-    },
-    /// Load data from the most recent response of a particular request recipe
-    Request {
-        recipe: RecipeId,
-        /// When should this request be automatically re-executed?
-        #[serde(default)]
-        trigger: ChainRequestTrigger,
-        #[serde(default)]
-        section: ChainRequestSection,
-    },
-    /// Prompt the user to select a value from a list
-    Select {
-        /// Descriptor to show to the user
-        message: Option<Template>,
-        /// List of options to choose from
-        options: SelectOptions,
-    },
-}
-
-/// Static or dynamic list of options for a select chain
-#[derive(Debug, Serialize, Deserialize)]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(untagged)]
-pub enum SelectOptions {
-    Fixed(Vec<Template>),
-    /// Render a template, then parse its output as a JSON array to get options
-    Dynamic(Template),
-}
-
-/// Test-only helpers
-#[cfg(any(test, feature = "test"))]
-impl ChainSource {
-    /// Build a new [Self::Command] variant from [command, ...args]
-    pub fn command<const N: usize>(cmd: [&str; N]) -> ChainSource {
-        ChainSource::Command {
-            command: cmd.into_iter().map(Template::from).collect(),
-            stdin: None,
-        }
-    }
-}
-
-/// The component of the response to use as the chain source
-#[derive(Debug, Default, Serialize, Deserialize)]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(rename_all = "snake_case", deny_unknown_fields)]
-pub enum ChainRequestSection {
-    #[default]
-    Body,
-    /// Pull a value from a response's headers. If the given header appears
-    /// multiple times, the first value will be used
-    Header(Template),
-}
-
-/// Define when a recipe with a chained request should auto-execute the
-/// dependency request.
-#[derive(Copy, Clone, Debug, Default, Serialize, Deserialize)]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(rename_all = "snake_case", deny_unknown_fields)]
-pub enum ChainRequestTrigger {
-    /// Never trigger the request. This is the default because upstream
-    /// requests could be mutating, so we want the user to explicitly opt into
-    /// automatic execution.
-    #[default]
-    Never,
-    /// Trigger the request if there is none in history
-    NoHistory,
-    /// Trigger the request if the last response is older than some
-    /// duration (or there is none in history)
-    Expire(#[serde(with = "cereal::serde_duration")] Duration),
-    /// Trigger the request every time the dependent request is rendered
-    Always,
-}
-
-/// Control how a JSONPath selector returns 0 vs 1 vs 2+ results
-#[derive(Copy, Clone, Debug, Default, Serialize, Deserialize)]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(rename_all = "snake_case", deny_unknown_fields)]
-pub enum SelectorMode {
-    /// 0 - Error
-    /// 1 - Single result, without wrapping quotes
-    /// 2 - JSON array
-    #[default]
-    Auto,
-    /// 0 - Error
-    /// 1 - Single result, without wrapping quotes
-    /// 2 - Error
-    Single,
-    /// 0 - JSON array
-    /// 1 - JSON array
-    /// 2 - JSON array
-    Array,
-}
-
-/// Trim whitespace from rendered output
-#[derive(Copy, Clone, Debug, Default, Serialize, Deserialize)]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(rename_all = "snake_case", deny_unknown_fields)]
-pub enum ChainOutputTrim {
-    /// Do not trim the output
-    #[default]
-    None,
-    /// Trim the start of the output
-    Start,
-    /// Trim the end of the output
-    End,
-    /// Trim the start and end of the output
-    Both,
-}
-
 impl Collection {
     /// Get the profile marked as `default: true`, if any. At most one profile
     /// can be marked as default.
@@ -611,9 +534,9 @@ impl slumber_util::Factory for Collection {
         };
         let profile = Profile::factory(());
         Collection {
+            name: None,
             recipes: by_id([recipe]).into(),
             profiles: by_id([profile]),
-            ..Collection::default()
         }
     }
 }
@@ -622,6 +545,7 @@ impl slumber_util::Factory for Collection {
 mod tests {
     use super::*;
     use indexmap::indexmap;
+    use itertools::Itertools;
     use rstest::rstest;
     use slumber_util::Factory;
 
@@ -676,4 +600,23 @@ mod tests {
         let expected = expected.and_then(|value| value.parse::<Mime>().ok());
         assert_eq!(recipe.mime(), expected);
     }
+
+    #[test]
+    fn test_query_iter() {
+        let recipe = Recipe {
+            query: indexmap! {
+                "param1".into() => ["value1.1", "value1.2"].into(),
+                "param2".into() => "value2.1".into(),
+            },
+            ..Recipe::factory(())
+        };
+        assert_eq!(
+            recipe.query_iter().collect_vec().as_slice(),
+            &[
+                ("param1", 0, &"value1.1".into()),
+                ("param1", 1, &"value1.2".into()),
+                ("param2", 0, &"value2.1".into()),
+            ]
+        );
+    }
 }
