@@ -2,10 +2,8 @@
 
 use super::*;
 use crate::{
-    collection::{Authentication, Chain, ChainSource, Collection, Profile},
-    test_util::{
-        TestPrompter, by_id, header_map, http_engine, invalid_utf8_chain,
-    },
+    collection::{Authentication, Profile},
+    test_util::{TestPrompter, by_id, header_map, http_engine, invalid_utf8},
 };
 use indexmap::{IndexMap, indexmap};
 use pretty_assertions::assert_eq;
@@ -13,12 +11,12 @@ use regex::Regex;
 use reqwest::{Body, StatusCode, header};
 use rstest::rstest;
 use serde_json::json;
-use slumber_util::{Factory, test_data_dir};
+use slumber_util::{Factory, assert_err, test_data_dir};
 use std::ptr;
 use wiremock::{Mock, MockServer, ResponseTemplate, matchers};
 
-/// Create a template context. Take a set of extra recipes and chains to
-/// add to the created collection
+/// Create a template context. Take a set of extra recipes to add to the created
+/// collection
 fn template_context(recipe: Recipe) -> TemplateContext {
     let profile_data = indexmap! {
         "host".into() => "http://localhost".into(),
@@ -28,39 +26,17 @@ fn template_context(recipe: Recipe) -> TemplateContext {
         "username".into() => "user".into(),
         "password".into() => "hunter2".into(),
         "token".into() => "tokenzzz".into(),
+        "test_data_dir".into() => test_data_dir().to_str().unwrap().into(),
+        "prompt".into() => "{{ prompt() }}".into(),
+        "error".into() => "{{ fake_fn() }}".into(),
     };
     let profile = Profile {
         data: profile_data,
         ..Profile::factory(())
     };
-    let profile_id = profile.id.clone();
-    let chains = [
-        Chain {
-            id: "text".into(),
-            source: ChainSource::Prompt {
-                message: None,
-                default: None,
-            },
-            ..Chain::factory(())
-        },
-        Chain {
-            // Invalid UTF-8
-            id: "binary".into(),
-            source: invalid_utf8_chain(test_data_dir()),
-            ..Chain::factory(())
-        },
-    ];
     TemplateContext {
-        collection: Collection {
-            name: None,
-            recipes: by_id([recipe]).into(),
-            profiles: by_id([profile]),
-            chains: by_id(chains),
-        }
-        .into(),
-        selected_profile: Some(profile_id.clone()),
         prompter: Box::new(TestPrompter::new(["first", "second"])),
-        ..TemplateContext::factory(())
+        ..TemplateContext::factory((by_id([profile]), by_id([recipe])))
     }
 }
 
@@ -72,29 +48,6 @@ fn seed(context: &TemplateContext, build_options: BuildOptions) -> RequestSeed {
     )
 }
 
-/// Create a mock HTTP server and return its URL
-async fn mock_server() -> String {
-    // Mock HTTP response
-    let server = MockServer::start().await;
-    let host = server.uri();
-    Mock::given(matchers::method("GET"))
-        .and(matchers::path("/get"))
-        .respond_with(
-            ResponseTemplate::new(StatusCode::OK).set_body_string("hello!"),
-        )
-        .mount(&server)
-        .await;
-    Mock::given(matchers::method("GET"))
-        .and(matchers::path("/redirect"))
-        .respond_with(
-            ResponseTemplate::new(StatusCode::MOVED_PERMANENTLY)
-                .insert_header("Location", format!("{host}/get")),
-        )
-        .mount(&server)
-        .await;
-    host
-}
-
 /// Make sure we only use the dangerous client when we really expect to.
 /// There's isn't an easy way to mock TLS errors, so the easiest way to
 /// test this is to just make sure [HttpEngine::get_client] returns the
@@ -124,17 +77,17 @@ fn test_get_client(
 async fn test_build_request(http_engine: HttpEngine) {
     let recipe = Recipe {
         method: HttpMethod::Post,
-        url: "{{host}}/users/{{user_id}}".into(),
-        query: vec![
-            ("mode".into(), "{{mode}}".into()),
-            ("fast".into(), "true".into()),
-        ],
+        url: "{{ host }}/users/{{ user_id }}".into(),
+        query: indexmap! {
+            "mode".into() => "{{ mode }}".into(),
+            "fast".into() => "true".into(),
+        },
         headers: indexmap! {
             // Leading/trailing newlines should be stripped
             "Accept".into() => "application/json".into(),
             "Content-Type".into() => "application/json".into(),
         },
-        body: Some("{\"group_id\":\"{{group_id}}\"}".into()),
+        body: Some("{\"group_id\":\"{{ group_id }}\"}".into()),
         ..Recipe::factory(())
     };
     let recipe_id = recipe.id.clone();
@@ -184,13 +137,11 @@ async fn test_build_request(http_engine: HttpEngine) {
 #[tokio::test]
 async fn test_build_url(http_engine: HttpEngine) {
     let recipe = Recipe {
-        url: "{{host}}/users/{{user_id}}".into(),
-        query: vec![
-            ("mode".into(), "{{mode}}".into()),
-            ("fast".into(), "true".into()),
-            ("fast".into(), "false".into()),
-            ("mode".into(), "user".into()),
-        ],
+        url: "{{ host }}/users/{{ user_id }}".into(),
+        query: indexmap! {
+            "mode".into() => ["{{ mode }}", "user"].into(),
+            "fast".into() => ["true", "false"].into(),
+        },
         ..Recipe::factory(())
     };
     let context = template_context(recipe);
@@ -199,21 +150,21 @@ async fn test_build_url(http_engine: HttpEngine) {
 
     assert_eq!(
         url.as_str(),
-        "http://localhost/users/1?mode=sudo&fast=true&fast=false&mode=user"
+        "http://localhost/users/1?mode=sudo&mode=user&fast=true&fast=false"
     );
 }
 
 /// Test building just a body. URL/query/headers should *not* be built.
 #[rstest]
 #[case::raw(
-    RecipeBody::Raw(r#"{"group_id":"{{group_id}}"}"#.into()),
+    RecipeBody::Raw(r#"{"group_id":"{{ group_id }}"}"#.into()),
     br#"{"group_id":"3"}"#
 )]
 #[case::json(
-    RecipeBody::json(json!({"group_id": "{{group_id}}"})).unwrap(),
+    RecipeBody::json(json!({"group_id": "{{ group_id }}"})).unwrap(),
     br#"{"group_id":"3"}"#,
 )]
-#[case::binary(RecipeBody::Raw("{{chains.binary}}".into()), b"\xc3\x28")]
+#[case::binary(RecipeBody::Raw(invalid_utf8()), b"\xc3\x28")]
 #[tokio::test]
 async fn test_build_body(
     http_engine: HttpEngine,
@@ -234,19 +185,19 @@ async fn test_build_body(
 #[rstest]
 #[case::basic(
     Authentication::Basic {
-        username: "{{username}}".into(),
-        password: Some("{{password}}".into()),
+        username: "{{ username }}".into(),
+        password: Some("{{ password }}".into()),
     },
     "Basic dXNlcjpodW50ZXIy"
 )]
 #[case::basic_no_password(
     Authentication::Basic {
-        username: "{{username}}".into(),
+        username: "{{ username }}".into(),
         password: None,
     },
     "Basic dXNlcjo="
 )]
-#[case::bearer(Authentication::Bearer("{{token}}".into()), "Bearer tokenzzz")]
+#[case::bearer(Authentication::Bearer { token: "{{ token }}".into() }, "Bearer tokenzzz")]
 #[tokio::test]
 async fn test_authentication(
     http_engine: HttpEngine,
@@ -292,27 +243,67 @@ async fn test_authentication(
 /// hypothetically vary from the request record.
 #[rstest]
 #[case::json(
-    RecipeBody::json(json!({"group_id": "{{group_id}}"})).unwrap(),
+    RecipeBody::json(json!({"group_id": "{{ group_id }}"})).unwrap(),
     None,
-    Some(b"{\"group_id\":\"3\"}".as_slice()),
+    Some(r#"{"group_id":"3"}"#),
     "^application/json$",
     &[],
 )]
 // Content-Type has been overridden by an explicit header
 #[case::json_content_type_override(
-    RecipeBody::json(json!({"group_id": "{{group_id}}"})).unwrap(),
+    RecipeBody::json(json!({"group_id": "{{ group_id }}"})).unwrap(),
     Some("text/plain"),
-    Some(br#"{"group_id":"3"}"#.as_slice()),
+    Some(r#"{"group_id":"3"}"#),
     "^text/plain$",
     &[],
 )]
+#[case::json_unpack(
+    // Single-chunk templates should get unpacked to the actual JSON value
+    // instead of returned as a string
+    RecipeBody::json(json!("{{ [1,2,3] }}")).unwrap(),
+    None,
+    Some("[1,2,3]"),
+    "^application/json$",
+    &[],
+)]
+#[case::json_no_unpack(
+    // This template doesn't get unpacked because it is multiple chunks
+    RecipeBody::json(json!("no: {{ [1,2,3] }}")).unwrap(),
+    None,
+    // Spaces are added because this uses the template Value stringification
+    // instead of serde_json stringification
+    Some(r#""no: [1, 2, 3]""#),
+    "^application/json$",
+    &[],
+)]
+#[case::json_string_from_file(
+    // JSON data is loaded as a string and NOT unpacked. file() returns bytes
+    // which automatically get interpreted as a string.
+    RecipeBody::json(json!(
+        "{{ file(concat([test_data_dir, '/data.json'])) | trim() }}"
+    )).unwrap(),
+    None,
+    Some(r#""{ \"a\": 1, \"b\": 2 }""#),
+    "^application/json$",
+    &[],
+)]
+#[case::json_from_file_parsed(
+    // Pipe to json_parse() to parse it
+    RecipeBody::json(json!(
+        "{{ file(concat([test_data_dir, '/data.json'])) | json_parse() }}"
+    )).unwrap(),
+    None,
+    Some(r#"{"a":1,"b":2}"#),
+    "^application/json$",
+    &[],
+)]
 #[case::form_urlencoded(
     RecipeBody::FormUrlencoded(indexmap! {
-        "user_id".into() => "{{user_id}}".into(),
-        "token".into() => "{{token}}".into()
+        "user_id".into() => "{{ user_id }}".into(),
+        "token".into() => "{{ token }}".into()
     }),
     None,
-    Some(b"user_id=1&token=tokenzzz".as_slice()),
+    Some("user_id=1&token=tokenzzz"),
     "^application/x-www-form-urlencoded$",
     &[],
 )]
@@ -321,14 +312,14 @@ async fn test_authentication(
 #[case::form_urlencoded_content_type_override(
     RecipeBody::FormUrlencoded(Default::default()),
     Some("text/plain"),
-    Some(b"".as_slice()),
+    Some(""),
     "^text/plain$",
     &[],
 )]
 #[case::form_multipart(
     RecipeBody::FormMultipart(indexmap! {
-        "user_id".into() => "{{user_id}}".into(),
-        "binary".into() => "{{chains.binary}}".into()
+        "user_id".into() => "{{ user_id }}".into(),
+        "binary".into() => invalid_utf8(),
     }),
     None,
     // multipart bodies are automatically turned into streams by reqwest,
@@ -343,7 +334,7 @@ async fn test_structured_body(
     http_engine: HttpEngine,
     #[case] body: RecipeBody,
     #[case] content_type: Option<&str>,
-    #[case] expected_body: Option<&'static [u8]>,
+    #[case] expected_body: Option<&'static str>,
     // For multipart bodies, the content type includes random content
     #[case] expected_content_type: Regex,
     #[case] extra_headers: &[(&str, &str)],
@@ -379,7 +370,12 @@ async fn test_structured_body(
             to match `{expected_content_type}`"
     );
     assert_eq!(
-        ticket.request.body().and_then(Body::as_bytes),
+        ticket
+            .request
+            .body()
+            .and_then(Body::as_bytes)
+            // We know all the bodies are UTF-8. This gives better errors
+            .map(|bytes| std::str::from_utf8(bytes).unwrap()),
         expected_body
     );
 
@@ -421,8 +417,8 @@ async fn test_override_authentication(http_engine: HttpEngine) {
         &context,
         BuildOptions {
             authentication: Some(Authentication::Basic {
-                username: "{{username}}".into(),
-                password: Some("{{password}}".into()),
+                username: "{{ username }}".into(),
+                password: Some("{{ password }}".into()),
             }),
             ..Default::default()
         },
@@ -488,14 +484,15 @@ async fn test_override_headers(http_engine: HttpEngine) {
 async fn test_override_query(http_engine: HttpEngine) {
     let recipe = Recipe {
         url: "http://localhost/url".into(),
-        query: vec![
+        query: indexmap! {
             // Overridden
-            ("mode".into(), "regular".into()),
+            "mode".into() => "regular".into(),
             // Excluded
-            ("fast".into(), "false".into()),
-            // Included
-            ("fast".into(), "true".into()),
-        ],
+            "fast".into() => [
+                "false", // Excluded
+                "true", // Included
+            ].into(),
+        },
         ..Recipe::factory(())
     };
     let context = template_context(recipe);
@@ -503,7 +500,7 @@ async fn test_override_query(http_engine: HttpEngine) {
         &context,
         BuildOptions {
             query_parameters: [
-                (0, BuildFieldOverride::Override("{{mode}}".into())),
+                (0, BuildFieldOverride::Override("{{ mode }}".into())),
                 (1, BuildFieldOverride::Omit),
             ]
             .into_iter()
@@ -525,14 +522,14 @@ async fn test_override_query(http_engine: HttpEngine) {
 #[tokio::test]
 async fn test_override_body_raw(http_engine: HttpEngine) {
     let recipe = Recipe {
-        body: Some(RecipeBody::Raw("{{username}}".into())),
+        body: Some(RecipeBody::Raw("{{ username }}".into())),
         ..Recipe::factory(())
     };
     let context = template_context(recipe);
     let seed = seed(
         &context,
         BuildOptions {
-            body: Some("{{password}}".into()),
+            body: Some("{{ password }}".into()),
             ..Default::default()
         },
     );
@@ -554,7 +551,7 @@ async fn test_override_body_raw(http_engine: HttpEngine) {
 async fn test_override_body_json(http_engine: HttpEngine) {
     let recipe = Recipe {
         body: Some(
-            RecipeBody::json(json!({"username": "{{username}}"})).unwrap(),
+            RecipeBody::json(json!({"username": "{{ username }}"})).unwrap(),
         ),
         ..Recipe::factory(())
     };
@@ -588,9 +585,9 @@ async fn test_override_body_form(http_engine: HttpEngine) {
         // This should implicitly set the content-type header
         body: Some(RecipeBody::FormUrlencoded(indexmap! {
             // Included
-            "user_id".into() => "{{user_id}}".into(),
+            "user_id".into() => "{{ user_id }}".into(),
             // Excluded
-            "token".into() => "{{token}}".into(),
+            "token".into() => "{{ token }}".into(),
             // Overridden
             "preference".into() => "large".into(),
         })),
@@ -631,15 +628,15 @@ async fn test_override_body_form(http_engine: HttpEngine) {
     );
 }
 
-/// Using the same chain in two different templates should be deduplicated,
-/// so that the chain is only computed once
+/// Using the same profile field in two different templates should be
+/// deduplicated, so that the expression is only evaluated once
 #[rstest]
 #[tokio::test]
-async fn test_chain_duplicate(http_engine: HttpEngine) {
+async fn test_profile_duplicate(http_engine: HttpEngine) {
     let recipe = Recipe {
         method: HttpMethod::Post,
-        url: "{{host}}/{{chains.text}}".into(),
-        body: Some("{{chains.text}}".into()),
+        url: "{{ host }}/{{ prompt }}/{{ prompt }}".into(),
+        body: Some("{{ prompt }}".into()),
         ..Recipe::factory(())
     };
     let context = template_context(recipe);
@@ -647,14 +644,41 @@ async fn test_chain_duplicate(http_engine: HttpEngine) {
     let seed = seed(&context, BuildOptions::default());
     let ticket = http_engine.build(seed, &context).await.unwrap();
 
-    let expected_url: Url = "http://localhost/first".parse().unwrap();
-    let expected_body = b"first";
+    let expected_url: Url = "http://localhost/first/first".parse().unwrap();
+    let expected_body = "first";
 
     let request = &ticket.request;
     assert_eq!(request.url(), &expected_url);
     assert_eq!(
-        request.body().and_then(Body::as_bytes),
-        Some(expected_body.as_slice())
+        request
+            .body()
+            .and_then(|body| std::str::from_utf8(body.as_bytes()?).ok()),
+        Some(expected_body)
+    );
+}
+
+/// If a profile field is rendered twice in two separate templates but the first
+/// call fails, the second should fail as well
+#[rstest]
+#[tokio::test]
+async fn test_profile_duplicate_error(http_engine: HttpEngine) {
+    let recipe = Recipe {
+        method: HttpMethod::Post,
+        url: "{{ host }}/{{ error }}".into(),
+        body: Some("{{ error }}".into()),
+        ..Recipe::factory(())
+    };
+    let recipe_id = recipe.id.clone();
+    let context = template_context(recipe);
+
+    let seed = RequestSeed::new(recipe_id, BuildOptions::default());
+    assert_err!(
+        http_engine
+            .build(seed, &context)
+            .await
+            // Include full error chain in the message
+            .map_err(anyhow::Error::from),
+        "fake_fn(): Unknown function"
     );
 }
 
@@ -662,9 +686,18 @@ async fn test_chain_duplicate(http_engine: HttpEngine) {
 #[rstest]
 #[tokio::test]
 async fn test_send_request(http_engine: HttpEngine) {
-    let host = mock_server().await;
+    // Mock HTTP response
+    let server = MockServer::start().await;
+    Mock::given(matchers::method("GET"))
+        .and(matchers::path("/get"))
+        .respond_with(
+            ResponseTemplate::new(StatusCode::OK).set_body_string("hello!"),
+        )
+        .mount(&server)
+        .await;
+
     let recipe = Recipe {
-        url: format!("{host}/get").as_str().into(),
+        url: format!("{host}/get", host = server.uri()).as_str().into(),
         ..Recipe::factory(())
     };
     let context = template_context(recipe);
@@ -708,7 +741,7 @@ async fn test_render_headers_strip() {
         // Leading/trailing newlines should be stripped
         headers: indexmap! {
             "Accept".into() => "application/json".into(),
-            "Host".into() => "\n{{host}}\n".into(),
+            "Host".into() => "\n{{ host }}\n".into(),
         },
         ..Recipe::factory(())
     };
@@ -745,11 +778,10 @@ fn test_trim_bytes(#[case] bytes: &[u8], #[case] expected: &[u8]) {
 async fn test_build_curl(http_engine: HttpEngine) {
     let recipe = Recipe {
         method: HttpMethod::Get,
-        query: vec![
-            ("mode".into(), "{{mode}}".into()),
-            ("fast".into(), "true".into()),
-            ("fast".into(), "false".into()),
-        ],
+        query: indexmap! {
+            "mode".into() => "{{ mode }}".into(),
+            "fast".into() => ["true", "false"].into(),
+        },
         headers: indexmap! {
             "Accept".into() => "application/json".into(),
             "Content-Type".into() => "application/json".into(),
@@ -771,20 +803,20 @@ async fn test_build_curl(http_engine: HttpEngine) {
 #[rstest]
 #[case::basic(
     Authentication::Basic {
-        username: "{{username}}".into(),
-        password: Some("{{password}}".into()),
+        username: "{{ username }}".into(),
+        password: Some("{{ password }}".into()),
     },
     "--user 'user:hunter2'",
 )]
 #[case::basic_no_password(
     Authentication::Basic {
-        username: "{{username}}".into(),
+        username: "{{ username }}".into(),
         password: None,
     },
     "--user 'user:'",
 )]
 #[case::bearer(
-    Authentication::Bearer("{{token}}".into()),
+    Authentication::Bearer { token: "{{ token }}".into() },
     "--header 'authorization: Bearer tokenzzz'",
 )]
 #[tokio::test]
@@ -809,21 +841,21 @@ async fn test_build_curl_authentication(
 /// Build a curl command with each possible type of body
 #[rstest]
 #[case::json(
-    RecipeBody::json(json!({"group_id": "{{group_id}}"})).unwrap(),
+    RecipeBody::json(json!({"group_id": "{{ group_id }}"})).unwrap(),
     r#"--json '{"group_id":"3"}'"#
 )]
 #[case::form_urlencoded(
     RecipeBody::FormUrlencoded(indexmap! {
-        "user_id".into() => "{{user_id}}".into(),
-        "token".into() => "{{token}}".into()
+        "user_id".into() => "{{ user_id }}".into(),
+        "token".into() => "{{ token }}".into()
     }),
     "--data-urlencode 'user_id=1' --data-urlencode 'token=tokenzzz'"
 )]
 #[case::form_multipart(
     // This doesn't support binary content because we can't pass it via cmd
     RecipeBody::FormMultipart(indexmap! {
-        "user_id".into() => "{{user_id}}".into(),
-        "token".into() => "{{token}}".into()
+        "user_id".into() => "{{ user_id }}".into(),
+        "token".into() => "{{ token }}".into()
     }),
     "-F 'user_id=1' -F 'token=tokenzzz'"
 )]
@@ -846,35 +878,36 @@ async fn test_build_curl_body(
     assert_eq!(command, expected_command);
 }
 
-/// By default, the engine will follow 3xx redirects
-#[rstest]
-#[tokio::test]
-async fn test_follow_redirects(http_engine: HttpEngine) {
-    let host = mock_server().await;
-    let recipe = Recipe {
-        url: format!("{host}/redirect").as_str().into(),
-        ..Recipe::factory(())
-    };
-    let context = template_context(recipe);
-    let seed = seed(&context, BuildOptions::default());
-
-    // Build+send the request
-    let ticket = http_engine.build(seed, &context).await.unwrap();
-    let exchange = ticket.send().await.unwrap();
-
-    // Should hit /redirect which redirects to /get, which returns the body
-    assert_eq!(exchange.response.status, StatusCode::OK);
-    assert_eq!(exchange.response.body.bytes().as_ref(), b"hello!");
-}
-
 /// Client should not follow redirects when the config field is disabled
+#[rstest]
+#[case::enabled(true, StatusCode::OK)]
+#[case::disabled(false, StatusCode::MOVED_PERMANENTLY)]
 #[tokio::test]
-async fn test_follow_redirects_disabled() {
+async fn test_follow_redirects(
+    #[case] follow_redirects: bool,
+    #[case] expected_status: StatusCode,
+) {
+    // Mock HTTP responses
+    let server = MockServer::start().await;
+    let host = server.uri();
+    Mock::given(matchers::method("GET"))
+        .and(matchers::path("/get"))
+        .respond_with(ResponseTemplate::new(StatusCode::OK))
+        .mount(&server)
+        .await;
+    Mock::given(matchers::method("GET"))
+        .and(matchers::path("/redirect"))
+        .respond_with(
+            ResponseTemplate::new(StatusCode::MOVED_PERMANENTLY)
+                .insert_header("Location", format!("{host}/get")),
+        )
+        .mount(&server)
+        .await;
+
     let http_engine = HttpEngine::new(&HttpEngineConfig {
-        follow_redirects: false,
+        follow_redirects,
         ..Default::default()
     });
-    let host = mock_server().await;
     let recipe = Recipe {
         url: format!("{host}/redirect").as_str().into(),
         ..Recipe::factory(())
@@ -886,5 +919,5 @@ async fn test_follow_redirects_disabled() {
     let ticket = http_engine.build(seed, &context).await.unwrap();
     let exchange = ticket.send().await.unwrap();
 
-    assert_eq!(exchange.response.status, StatusCode::MOVED_PERMANENTLY);
+    assert_eq!(exchange.response.status, expected_status);
 }
