@@ -4,23 +4,26 @@
 //! `slumber` crate version. If you choose to depend directly on this crate, you
 //! do so at your own risk of breakage.
 
+mod common;
 mod insomnia;
 mod openapi;
 mod rest;
+mod v3;
 
+pub use insomnia::from_insomnia;
+pub use openapi::from_openapi;
+pub use rest::from_rest;
+pub use v3::from_v3;
+
+use anyhow::Context;
+use futures::TryFutureExt;
+use reqwest::{Response, Url};
 use std::{
     convert::Infallible,
     ffi::OsStr,
     path::{Path, PathBuf},
     str::FromStr,
 };
-
-use anyhow::Context;
-use futures::TryFutureExt;
-pub use insomnia::from_insomnia;
-pub use openapi::from_openapi;
-use reqwest::{Response, Url};
-pub use rest::from_rest;
 use tokio::{
     fs,
     io::{self, AsyncReadExt, BufReader},
