@@ -27,7 +27,7 @@ use ratatui::{
     layout::{Alignment, Constraint, Layout},
     style::Style,
     text::{Line, Span},
-    widgets::{Paragraph, block::Title},
+    widgets::Paragraph,
 };
 use serde::{Deserialize, Serialize};
 use slumber_config::Action;
@@ -91,7 +91,7 @@ impl Draw for ExchangePane {
         // If a recipe is selected, history is available so show the hint
         if matches!(self.state, State::Content { .. }) {
             let text = input_engine.add_hint("History", Action::History);
-            block = block.title(Title::from(text).alignment(Alignment::Right));
+            block = block.title(Line::from(text).alignment(Alignment::Right));
         }
         frame.render_widget(&block, metadata.area());
         let area = block.inner(metadata.area());
@@ -138,6 +138,7 @@ pub enum ExchangePaneEvent {
 /// Inner state for the exchange pane. This contains all the empty states, as
 /// well as one variant for the populated state
 #[derive(Debug, Default)]
+#[expect(clippy::large_enum_variant)]
 enum State {
     /// Recipe list is empty
     #[default]
@@ -448,6 +449,7 @@ impl Draw for ExchangePaneContent {
 
 /// Various request states that can appear under the tab bar
 #[derive(Debug)]
+
 enum ExchangePaneContentState {
     Building,
     BuildError {
