@@ -11,6 +11,7 @@ use crate::{
             modal::Modal,
         },
         component::{
+            collection_select::CollectionSelect,
             exchange_pane::{ExchangePane, ExchangePaneEvent},
             help::HelpModal,
             misc::DeleteRecipeRequestsModal,
@@ -261,6 +262,7 @@ impl EventHandler for PrimaryView {
                 Action::SelectResponse => {
                     self.selected_pane.get_mut().select(&PrimaryPane::Exchange);
                 }
+                Action::SelectCollection => CollectionSelect::new().open(),
 
                 // Toggle fullscreen
                 Action::Fullscreen => {
@@ -461,6 +463,7 @@ enum FullscreenMode {
 /// Menu actions available in all contexts
 #[derive(Copy, Clone, Debug, Display, EnumIter)]
 enum GlobalMenuAction {
+    /// Open the current collection file in an external editor
     #[display("Edit Collection")]
     EditCollection,
 }
@@ -492,11 +495,11 @@ mod tests {
             test_util::TestComponent, util::persistence::DatabasePersistedStore,
         },
     };
-    use crossterm::event::KeyCode;
     use persisted::PersistedStore;
     use rstest::rstest;
     use slumber_core::http::BuildOptions;
     use slumber_util::assert_matches;
+    use terminput::KeyCode;
 
     /// Create component to be tested
     fn create_component<'term>(
@@ -558,11 +561,7 @@ mod tests {
 
         harness.clear_messages(); // Clear init junk
 
-        component
-            .int()
-            .open_actions()
-            .send_key(KeyCode::Enter) // Select first action - Edit Collection
-            .assert_empty();
+        component.int().action("Edit Collection").assert_empty();
         // Event should be converted into a message appropriately
         assert_matches!(harness.pop_message_now(), Message::CollectionEdit);
     }
@@ -576,9 +575,7 @@ mod tests {
         component
             .int()
             .send_key(KeyCode::Char('l')) // Select recipe list
-            .open_actions()
-            // Copy URL
-            .send_keys([KeyCode::Down, KeyCode::Enter])
+            .action("Copy URL")
             .assert_empty();
 
         assert_matches!(harness.pop_message_now(), Message::CopyRequestUrl);
@@ -593,9 +590,7 @@ mod tests {
         component
             .int()
             .send_key(KeyCode::Char('l')) // Select recipe list
-            .open_actions()
-            // Copy as cURL
-            .send_keys([KeyCode::Down, KeyCode::Down, KeyCode::Enter])
+            .action("Copy as cURL")
             .assert_empty();
 
         assert_matches!(harness.pop_message_now(), Message::CopyRequestCurl);
